from alkh.dump_data import take_it_offline
from alkh.print_utils import print_context
from alkh.analyze_call_graph import analyze


version_info = (0, 3, 0)
__version__ = '.'.join(map(str, version_info))
